:- ensure_loaded(['file_io','lib/list','lib/number']).

conflict(File, Conflicts) :-
	file_2terms(File, Terms0),
	terms_2headGoals(Terms0, Terms),
	list_list_scheme(Terms, Protos, X^Y^prototyp(X,Y)),
	setof(P, A0^A1^(list_member(Protos,P-A0),list_member(Protos,P-A1),A0=\=A1), Conflicts).

prototyp(head_goals(H,_Gs), Name-Arity) :-
	functor(H, Name, Arity).
prototyp(head(H), Name-Arity) :-
	functor(H, Name, Arity).


slowdown(PathPrefix, FileName) :-
	lists_appended([PathPrefix,"trans/",FileName], FileIn0),
	atom_chars(FileIn, FileIn0),
	int_for(I, 0, 32),
	number_chars(I, IChars),
	lists_appended([PathPrefix,"reg",IChars,"/",FileName], FileOut0),
	atom_chars(FileOut, FileOut0),
	file_trans_regs(FileIn, FileOut, I),
	nl,write(I),
	fail.
	


file_trans_regs(File0, File, Regs) :-
	file_2terms(File0, Terms0),
	terms_2headGoals(Terms0, Terms1),
	terms_trans_regs(Terms1, Terms2, Regs),
	headGoals_2terms(Terms2, Terms),
	terms_2file(Terms, File).


terms_trans_regs(Terms, Trans, Regs) :-
	list_list_argument_scheme(Terms, Trans, Regs, X^Y^R^term_trans_regs(X,Y,R)).

term_trans_regs(head_goals(H0,Gs0), head_goals(H,Gs), Regs) :-
	head_trans_regs(H0, H, Regs),
	goals_trans_regs(Gs0, Gs, Regs).
term_trans_regs(head(H0), head(H), Regs) :-
	head_trans_regs(H0, H, Regs).


head_trans_regs(H0, H, Regs) :-
	headGoal_trans_regs_args(H0, H, Regs, _Args).

goals_trans_regs(Gs0, ['VOID'(Void)|Gs], Regs) :-
	list_list_argument_scheme(Gs0, Gs, Regs-Void, G0^G^(R-V)^goal_trans_regs_void(G0,G,R,V)).

goal_trans_regs_void(G0, G, Regs, Void) :-
	headGoal_trans_regs_args(G0, G, Regs, Args),	
	list_argument_scheme(Args, Void, A^A^true).


headGoal_trans_regs_args(-HG0, HG, _Regs, AddArgs) :-
	!,
	HG = HG0,
	AddArgs = [].
headGoal_trans_regs_args(HG0, HG, Regs, AddArgs) :-
	functor(HG0, Name, Arity),
	HG0 =.. [Name|Args],
	atom_chars(Name, NameChars),
	number_chars(Arity, ArityChars),
	lists_appended([NameChars,"__",ArityChars], NewFunctorChars),
	atom_chars(NewFunctor, NewFunctorChars),
	AddRegs is max(0, Regs-Arity),
	length(AddArgs, AddRegs),
	list_list_appended(Args, AddArgs, NewArgs),  
	HG =.. [NewFunctor|NewArgs].







